
(*############################*)
 IMPLEMENTATION MODULE DynStr0;         (* v1.1-032404 *)
(*############################*)
                                       (* by Thomas Breeden, tmb@virginia.edu *)
(*
        Dynamically allocated string processing: Basic procedures.
*)

(*
Copyright (C) 1987-2004 Thomas Breeden.

Permission to use, copy, modify and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appear in all copies and that both the copyright
notice and this permission notice appear in supporting documentation.
Thomas Breeden makes no representations about the suitability of this software
for any purpose. It is provided "as is" without express or implied warranty.
*)

(*?D-*) (* dynamic array copying off *)
(*?R-*)  (* run time checks off *)

FROM Debugging0 IMPORT Assert;

FROM Storage    IMPORT ALLOCATE, DEALLOCATE;
FROM Str0       IMPORT StrSet;

(*
NOTE
     1. Compiler's idea of the size of .str field ARRAY can cause problems.

VERSION HISTORY

v1.0-012187   -First version

    -111489   -Added $D- (Benchmark AmigaDOS) compiler switch => No copying
               of non-VAR dynamic arrays. BE CAREFUL.

    -052696   -Changed HALT to Assert().

    -031298   -Importing Assert from Debugging instead of Assertions.

    -041799   -ISO

    -052399   -bugfix: 3 occurances of VAL(INTEGER, len-1) changed to
               VAL(INTEGER, len)-1.

    -080501   -Changed all StrLen() to LENGTH()
              - Documented that dStrAsgE() and dStrAsgC() will work correctly
                if there are embedded 0C chars in the input DynStr.

    -051603   -Changed dStrCreate() to (correctly) return with .str = NIL if the
               allocate fails. dStrCreateCheck() is now simply a call to dStrCreate().

v1.1-032404   -Changed and documented dStrDispose() to handle ok a call if .str is NIL.
*)

(*---------------------------------------*)
 PROCEDURE Minimum(A,B:CARDINAL):CARDINAL;
(*---------------------------------------*)

BEGIN

IF B < A THEN
   RETURN B
ELSE
   RETURN A
END;

END Minimum;

(*================================================*)
 PROCEDURE dStrCreate(VAR s:DynStr; size:CARDINAL);
(*================================================*)
   (* Create a new dynamic string *)

(* NOTE: 1.Always alloc one extra byte to insure that we can put a 0C
           at the end of the string.
*)

BEGIN WITH s DO

Assert(size <= MaxSiz, "dStrCreate");

siz := size;
len := 0;

ALLOCATE(str, siz+1);   (* extra one for assured NULL *)
IF str # NIL THEN
   str^[0] := 0C;
END;

END END dStrCreate;

(*==========================================================================*)
 PROCEDURE dStrCreateCheck(VAR s:DynStr; size:CARDINAL; VAR Success:BOOLEAN);
(*==========================================================================*)
   (* Create a new dynamic string, checking for available memory first *)

BEGIN

dStrCreate(s, size);
Success := (s.str # NIL);

END dStrCreateCheck;

(*==================================*)
 PROCEDURE dStrDispose(VAR s:DynStr);
(*==================================*)
   (* Dispose of dynamic string *)

BEGIN WITH s DO

IF str # NIL THEN
   DEALLOCATE(str, siz+1);
END;
siz := 0; len := 0;
(* str := NIL -> done by DEALLOCATE *)

END END dStrDispose;

(*===========================================*)
 PROCEDURE dStrAsgE(s1:DynStr; VAR s2:DynStr);
(*===========================================*)
   (* Assign to an existing dynamic string *)

VAR i   :INTEGER;

BEGIN WITH s2 DO

len := Minimum(s1.len, siz);
FOR i := 0 TO VAL(INTEGER, len)-1 DO
   str^[i] := s1.str^[i];
END;

str^[len] := 0C;

END END dStrAsgE;

(*===========================================*)
 PROCEDURE dStrAsgC(s1:DynStr; VAR s2:DynStr);
(*===========================================*)
   (* Assign to an non-existing dynamic string, create it smallest possible *)

BEGIN

dStrCreate(s2, s1.len);
dStrAsgE(s1, s2);

END dStrAsgC;

(*=================================================*)
 PROCEDURE dStrInE(s1:ARRAY OF CHAR; VAR s2:DynStr);
(*=================================================*)
    (* Assign to an existing DynStr from a "Str0" format string (literal) *)

VAR i   :INTEGER;

BEGIN WITH s2 DO

len := Minimum(LENGTH(s1), siz);
FOR i := 0 TO VAL(INTEGER, len)-1 DO
   str^[i] := s1[i];
END;

str^[len] := 0C;

END END dStrInE;

(*=================================================*)
 PROCEDURE dStrInC(s1:ARRAY OF CHAR; VAR s2:DynStr);
(*=================================================*)
    (* Assign to a non-existing DynStr from a "Str0" format string (literal) *)

BEGIN

dStrCreate(s2, LENGTH(s1));
dStrInE(s1, s2);

END dStrInC;

(*=================================================*)
 PROCEDURE dStrOut(s1:DynStr; VAR s2:ARRAY OF CHAR);
(*=================================================*)
    (* Assign to a "Str0" format string (literal) from a dynamic string *)

VAR i      :INTEGER;
    length :CARDINAL;

BEGIN WITH s1 DO

length := Minimum(s1.len, HIGH(s2)+1);
FOR i := 0 TO VAL(INTEGER, length)-1 DO
   s2[i] := str^[i];
END;
StrSet(s2, length);

END END dStrOut;

(*===================================*)
 PROCEDURE dStrLen(s:DynStr):CARDINAL;
(*===================================*)

BEGIN

RETURN s.len;

END dStrLen;

END DynStr0.

