
(*############################*)
 IMPLEMENTATION MODULE DynStr1;         (* v1.4-052399 for Benchmark *)
(*############################*)
                                       (* by Thomas Breeden, tmb@virginia.edu *)
(*
        Dynamically allocated string processing: Common procedures.
*)

(*
Copyright (C) 1987-2001 Thomas Breeden.

Permission to use, copy, modify and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appear in all copies and that both the copyright
notice and this permission notice appear in supporting documentation.
Thomas Breeden makes no representations about the suitability of this software
for any purpose. It is provided "as is" without express or implied warranty.
*)

(*?D-*) (* dynamic array copying = unnecessary *)
(*?R-*)  (* run time checks off *)

FROM DynStr0 IMPORT DynStr, dStrInC, dStrInE, dStrCreate, dStrDispose;

VAR  dstemp :DynStr;

(* VERSION HISTORY

v1.0-012187

v1.1-061789 for Benchmark
                -Now correctly setting a 0C byte after the end of the
                 string for dStrCat, dStrPre, and dStrSeg.

    -072289     -Changed INTEGER(len-1) to INTEGER(len)-1

v1.2-091695     -Added dStrLower().
                -Removed IF test for 'a' to 'z' in dStrCap().

v1.3-051296     -added dStr0Cat().

v1.4-052697     -added dStr0Pos, dStrPos0, dStr0Cmp
    -060197     -added dStr0Pre

    -052399     -bugfix: for dStrCap and dStrLower, changed VAL(INTEGER, len-1) to
                 VAL(INTEGER, len)-1

    -080501     -Changed all StrLen() to LENGTH()
                - Documented that dStrCat() and dStrPre() will work correctly
                  if there are embedded 0C chars in the input DynStr.
*)

(*---------------------------------------*)
 PROCEDURE Minimum(A,B:CARDINAL):CARDINAL;
(*---------------------------------------*)
BEGIN

IF A < B THEN RETURN A ELSE RETURN B END;

END Minimum;

(*=============================*)
 PROCEDURE dStrCap(VAR s:DynStr);
(*=============================*)

      (* Capitalize all alphabetics in a string *)

VAR i:INTEGER;

BEGIN WITH s DO

FOR i := 0 TO VAL(INTEGER, len)-1 DO                    (* MaxSiz < 32767 *)
   (*IF (str^[i] >= 'a') AND (str^[i] <= 'z') THEN*)
      str^[i] := CAP(str^[i]);
   (*END;*)
END;

END END dStrCap;

(*================================*)
 PROCEDURE dStrLower(VAR s:DynStr);
(*================================*)

      (* Lower case all alphabetics in a string *)

VAR i:INTEGER;

BEGIN WITH s DO

FOR i := 0 TO VAL(INTEGER, len)-1 DO                    (* MaxSiz < 32767 *)
   IF (str^[i] >= 'A') AND (str^[i] <= 'Z') THEN
      str^[i] := CHR(ORD(str^[i])+32);
   END;
END;

END END dStrLower;

(*==============================*)
 PROCEDURE dStrClr(VAR s:DynStr);
(*==============================*)

BEGIN WITH s DO

len := 0;
str^[0] := 0C;

END END dStrClr;

(*==================================================*)
 PROCEDURE dStrCat(Suffix:DynStr; VAR OutStr:DynStr);
(*==================================================*)
      (* Concatenate one string onto another *)

VAR  SuffixLen  :INTEGER;
     OutStrLen  :INTEGER;
     CharsMoved :INTEGER;
     Deficit    :INTEGER;

BEGIN WITH OutStr DO

OutStrLen := len;
SuffixLen := Suffix.len;

Deficit :=  OutStrLen + SuffixLen - VAL(INTEGER, siz);
IF Deficit > 0 THEN
   DEC(SuffixLen, Deficit);
END;

CharsMoved := 0;
WHILE CharsMoved < SuffixLen DO
   str^[CharsMoved + OutStrLen] := Suffix.str^[CharsMoved];
   INC(CharsMoved);
END;

len := OutStrLen + CharsMoved;
str^[len] := 0C;                   (* always available since siz+1 *)

END END dStrCat;

(*==========================================================*)
 PROCEDURE dStr0Cat(Suffix:ARRAY OF CHAR; VAR OutStr:DynStr);
(*==========================================================*)
     (* Concatenate one string in "Str0" format onto existing DynStr *)

VAR  SuffixLen  :INTEGER;
     OutStrLen  :INTEGER;
     CharsMoved :INTEGER;
     Deficit    :INTEGER;

BEGIN WITH OutStr DO

OutStrLen := len;
SuffixLen := LENGTH(Suffix);

Deficit :=  OutStrLen + SuffixLen - VAL(INTEGER, siz);
IF Deficit > 0 THEN
   DEC(SuffixLen, Deficit);
END;

CharsMoved := 0;
WHILE CharsMoved < SuffixLen DO
   str^[CharsMoved + OutStrLen] := Suffix[CharsMoved];
   INC(CharsMoved);
END;

len := OutStrLen + CharsMoved;
str^[len] := 0C;                   (* always available since siz+1 *)

END END dStr0Cat;

(*======================================*)
 PROCEDURE dStrCmp(s1,s2:DynStr):INTEGER;
(*======================================*)
      (* Compare two strings *)
      (* Returns 0 for equality, -1 if S1 < S2, +1 if S1 > S2 *)

(*

OUTPUT
     <function value>    +1 = S1 > S2
                          0 = equality
                         -1 = S1 < S2
*)

VAR
     MinLen,
     i          :CARDINAL;

BEGIN WITH s1 DO

MinLen := Minimum(len, s2.len);

i := 0;
LOOP
   IF (i = MinLen) OR (str^[i] # s2.str^[i]) THEN
      EXIT;
   END;
   INC(i);
END;

IF i = MinLen THEN
   IF len < s2.len THEN
      RETURN -1
   ELSIF len > s2.len THEN
      RETURN 1
   ELSE
      RETURN 0
   END;
ELSE
   IF str^[i] > s2.str^[i] THEN
      RETURN 1
   ELSE
      RETURN -1
   END;
END;

END END dStrCmp;

(*=======================================================*)
 PROCEDURE dStr0Cmp(s1:ARRAY OF CHAR; s2:DynStr):INTEGER;
(*=======================================================*)

VAR  iRes :INTEGER;

BEGIN

IF dstemp.siz < LENGTH(s1) THEN
   dStrDispose(dstemp);
   dStrInC(s1, dstemp);
ELSE
   dStrInE(s1, dstemp);
END;

iRes := dStrCmp(dstemp, s2);

IF dstemp.siz > 160 THEN
   dStrDispose(dstemp);
   dStrCreate(dstemp, 20);
END;

RETURN iRes;

END dStr0Cmp;

(*=========================================================*)
 PROCEDURE dStrPos(s,Pattern:DynStr; Start:INTEGER):INTEGER;
(*=========================================================*)
      (* Find the position of a substring in a string *)
      (* Start looking at position Start *)
      (* Returns < 0 if not a substring *)

(*
NOTE
        1.Nothing is ever found within the nullstr.
        2.Nothing is ever found if Start is beyond the end of the
          searched string.
        3.Otherwise, the nullstr is always found as a Pattern at Start.
*)

VAR  sLen,
     PatternLen,
     SearchLimit,
     PatternLimit,
     i,j                :INTEGER;
     Equal              :BOOLEAN;

BEGIN WITH s DO

IF Start < 0 THEN
   Start := 0
END;

PatternLen := Pattern.len;
PatternLimit := PatternLen-1;
sLen := len;
SearchLimit :=  sLen - PatternLen;

IF (Start+PatternLen) > sLen THEN
   RETURN -1
ELSIF PatternLen = 0 THEN
   IF Start < sLen THEN
      RETURN Start
   ELSE RETURN -1
   END;
ELSE
   i := Start-1;
   REPEAT
      INC(i);
      j := -1;
      REPEAT
        INC(j);
        Equal := (str^[i+j] = Pattern.str^[j]);
      UNTIL NOT Equal OR (j = PatternLimit);
   UNTIL Equal OR (i = SearchLimit);
   IF NOT Equal THEN
      RETURN -1
   ELSE RETURN i
   END;
END;

END END dStrPos;

(*===========================================================================*)
 PROCEDURE dStr0Pos(s:DynStr; Pattern:ARRAY OF CHAR; Start:INTEGER) :INTEGER;
(*===========================================================================*)
     (* Find the position of a "Str0" fromat substring in a DynStr *)

VAR  iRes :INTEGER;

BEGIN

IF dstemp.siz < LENGTH(Pattern) THEN
   dStrDispose(dstemp);
   dStrInC(Pattern, dstemp);
ELSE
   dStrInE(Pattern, dstemp);
END;

iRes := dStrPos(s, dstemp, Start);

IF dstemp.siz > 160 THEN
   dStrDispose(dstemp);
   dStrCreate(dstemp, 20);
END;

RETURN iRes;

END dStr0Pos;

(*===========================================================================*)
 PROCEDURE dStrPos0(s:ARRAY OF CHAR; Pattern:DynStr; Start:INTEGER) :INTEGER;
(*===========================================================================*)
     (* Find the position of a Dynstr in a "Str0" format string *)

VAR  iRes :INTEGER;

BEGIN

IF dstemp.siz < LENGTH(s) THEN
   dStrDispose(dstemp);
   dStrInC(s, dstemp);
ELSE
   dStrInE(s, dstemp);
END;

iRes := dStrPos(dstemp, Pattern, Start);

IF dstemp.siz > 160 THEN
   dStrDispose(dstemp);
   dStrCreate(dstemp, 20);
END;

RETURN iRes;

END dStrPos0;

(*==================================================*)
 PROCEDURE dStrPre(Prefix:DynStr; VAR OutStr:DynStr);
(*==================================================*)
      (* Prefix a string onto a string *)

VAR  PrefixLen,
     OutStrLen,
     i,
     CharsToMove,
     Deficit        :INTEGER;

BEGIN WITH OutStr DO

PrefixLen := Prefix.len;
OutStrLen := len;

CharsToMove := OutStrLen;
Deficit := (PrefixLen + OutStrLen) - VAL(INTEGER, siz);
IF Deficit > 0 THEN
   DEC(CharsToMove, Deficit);
END;

len := PrefixLen + CharsToMove;

i := CharsToMove - 1;
WHILE i >= 0 DO
   str^[i+PrefixLen] := str^[i];
   DEC(i);
END;

CharsToMove := PrefixLen;
Deficit := PrefixLen - VAL(INTEGER, siz);
IF Deficit > 0 THEN
   DEC(CharsToMove, Deficit);
END;

i := CharsToMove-1;
WHILE i >= 0 DO
   str^[i] := Prefix.str^[i];
   DEC(i);
END;

str^[len] := 0C;                   (* always available since siz+1 *)

END END dStrPre;

(*============================================================*)
 PROCEDURE dStr0Pre(Prefix:ARRAY OF CHAR; VAR OutStr:DynStr);
(*============================================================*)

VAR  PrefixLen,
     OutStrLen,
     i,
     CharsToMove,
     Deficit        :INTEGER;

BEGIN WITH OutStr DO

PrefixLen := LENGTH(Prefix);
OutStrLen := len;

CharsToMove := OutStrLen;
Deficit := (PrefixLen + OutStrLen) - VAL(INTEGER, siz);
IF Deficit > 0 THEN
   DEC(CharsToMove, Deficit);
END;

len := PrefixLen + CharsToMove;

i := CharsToMove - 1;
WHILE i >= 0 DO
   str^[i+PrefixLen] := str^[i];
   DEC(i);
END;

CharsToMove := PrefixLen;
Deficit := PrefixLen - VAL(INTEGER, siz);
IF Deficit > 0 THEN
   DEC(CharsToMove, Deficit);
END;

i := CharsToMove-1;
WHILE i >= 0 DO
   str^[i] := Prefix[i];
   DEC(i);
END;

str^[len] := 0C;                   (* always available since siz+1 *)

END END dStr0Pre;

(*=================================================*)
 PROCEDURE dStrSeg(InStr:DynStr; Start,Stop:INTEGER;
                   VAR OutStr:DynStr);
(*=================================================*)
      (* Assign a segment of one string to another *)

(*
NOTE
        1.Always ok to assign a segment to itself.

*)

VAR
   i,
   InStrLen,
   EndLoop    :INTEGER;

BEGIN WITH OutStr DO

InStrLen := InStr.len;
i := 0;

IF Start < 0 THEN
   Start := 0;
END;

IF (Start < InStrLen) AND (Stop >= 0) THEN

   IF Stop >= InStrLen THEN
      Stop := InStrLen-1;
   END;

   EndLoop := Stop - Start;
   IF EndLoop >= VAL(INTEGER, siz) THEN
      EndLoop := siz-1;
   END;

   WHILE i <= EndLoop DO
      str^[i] := InStr.str^[i+Start];
      INC(i);
   END;

END;

len := i;
str^[len] := 0C;                   (* always available since siz+1 *)

END END dStrSeg;

(*-------------------------*)
 BEGIN (* module init code *)
(*-------------------------*)

dStrCreate(dstemp, 20);

END DynStr1.

