
(*#############################*)
 IMPLEMENTATION MODULE DynStr1b;            (* $VER: DynStr1b.mod 0.0 (15.1.4) *)
(*#############################*)
                                       (* by Thomas Breeden, tmb@virginia.edu *)
(*
    Dynamically allocated string processing: some Strings module flavor procs
*)

(*
Copyright (C) 2003-2004 Thomas Breeden.

Permission to use, copy, modify and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appear in all copies and that both the copyright
notice and this permission notice appear in supporting documentation.
Thomas Breeden makes no representations about the suitability of this software
for any purpose. It is provided "as is" without express or implied warranty.
*)

(*FROM Debugging0 IMPORT DebugPause;*)

FROM DynStr0 IMPORT DynStr;
FROM MaxMin  IMPORT maxi, mini;

(* VERSION HISTORY
     v0.0 (21.7.3)   - first

          (15.1.4)   - Stony Brook requires a CARDINAL for DEC(,) second parameter
*)

(*======================================================*)
 PROCEDURE dStrDel(VAR str:DynStr; Start, Stop:INTEGER);
(*======================================================*)
    (* delete these chars from the existing dyn string *)

VAR  i, j, l :INTEGER;

BEGIN

IF Start < 0 THEN
   i := 0;
ELSE
   i := Start;
END;

l := str.len;
IF Stop >= l THEN
   j := l;
ELSE
   j := Stop+1;
END;

IF i < j THEN

   DEC(str.len, VAL(CARDINAL, j-i));

   WHILE j < l DO
      str.str^[i] := str.str^[j];
      INC(i);
      INC(j);
   END

END;

str.str^[str.len] := 0C;

END dStrDel;

(*=================================================================*)
 PROCEDURE dStrIns(InStr:DynStr; Start:INTEGER; VAR OutStr:DynStr);
(*=================================================================*)
    (* Inserts source into destination at position startPos               *)
    (* If Start pos is beyond end of InStr, chars are appended at the end *)
    (* If siz of OutStr is too small, chars are truncated silently        *)

VAR  ShiftPos,
     NumShift,
     NumOffEnd,
     NumInserted  :INTEGER;
     i, j, k      :INTEGER;

BEGIN

IF Start < 0 THEN
   Start := 0;
ELSIF Start >= VAL(INTEGER, OutStr.len) THEN
   Start := OutStr.len;
END;

NumOffEnd := 0;
NumShift := VAL(INTEGER, OutStr.len) - Start;

IF NumShift > 0 THEN
   ShiftPos := Start + VAL(INTEGER, InStr.len);
   (*DebugPause("NumShift", NumShift);*)
   (*DebugPause("ShiftPos", ShiftPos);*)

   NumOffEnd := mini(NumShift-(VAL(INTEGER, OutStr.siz)-ShiftPos), NumShift);

   (*DebugPause("NumOffEnd", NumOffEnd);*)
   IF NumOffEnd > 0 THEN
      DEC(NumShift, NumOffEnd);
      (*DebugPause("NumShift", NumShift);*)
   ELSE
      NumOffEnd := 0;
   END;

   j := ShiftPos+NumShift-1;
   k := Start+NumShift-1;
   FOR i := NumShift-1 TO 0 BY -1 DO
      OutStr.str^[j] := OutStr.str^[k];
      DEC(j);
      DEC(k);
   END;
END;
DEC(OutStr.len, VAL(CARDINAL, NumOffEnd));

NumInserted := InStr.len;
NumOffEnd := mini(NumInserted-(VAL(INTEGER, OutStr.siz)-Start), NumInserted);
(*DebugPause("NumOffEnd", NumOffEnd);*)
IF NumOffEnd > 0 THEN
   DEC(NumInserted, NumOffEnd);
END;

k := Start;
i := 0;
FOR i := 0 TO NumInserted-1 DO
   OutStr.str^[k] := InStr.str^[i];
   INC(k);
END;
INC(OutStr.len, VAL(CARDINAL, NumInserted));

OutStr.str^[OutStr.len] := 0C;

END dStrIns;

(*=========================================================================*)
 PROCEDURE dStr0Ins(InStr:ARRAY OF CHAR; Start:INTEGER; VAR OutStr:DynStr);
(*=========================================================================*)

VAR  ShiftPos,
     NumShift,
     NumOffEnd,
     NumInserted  :INTEGER;
     i, j, k      :INTEGER;

BEGIN

IF Start < 0 THEN
   Start := 0;
ELSIF Start >= VAL(INTEGER, OutStr.len) THEN
   Start := OutStr.len;
END;

NumOffEnd := 0;
NumShift := VAL(INTEGER, OutStr.len) - Start;

IF NumShift > 0 THEN
   ShiftPos := Start + VAL(INTEGER, LENGTH(InStr));
   (*DebugPause("NumShift", NumShift);*)
   (*DebugPause("ShiftPos", ShiftPos);*)

   NumOffEnd := mini(NumShift-(VAL(INTEGER, OutStr.siz)-ShiftPos), NumShift);

   (*DebugPause("NumOffEnd", NumOffEnd);*)
   IF NumOffEnd > 0 THEN
      DEC(NumShift, NumOffEnd);
      (*DebugPause("NumShift", NumShift);*)
   ELSE
      NumOffEnd := 0;
   END;

   j := ShiftPos+NumShift-1;
   k := Start+NumShift-1;
   FOR i := NumShift-1 TO 0 BY -1 DO
      OutStr.str^[j] := OutStr.str^[k];
      DEC(j);
      DEC(k);
   END;
END;
DEC(OutStr.len, VAL(CARDINAL, NumOffEnd));

NumInserted := LENGTH(InStr);
NumOffEnd := mini(NumInserted-(VAL(INTEGER, OutStr.siz)-Start), NumInserted);
(*DebugPause("NumOffEnd", NumOffEnd);*)
IF NumOffEnd > 0 THEN
   DEC(NumInserted, NumOffEnd);
END;

k := Start;
i := 0;
FOR i := 0 TO NumInserted-1 DO
   OutStr.str^[k] := InStr[i];
   INC(k);
END;
INC(OutStr.len, VAL(CARDINAL, NumInserted));

OutStr.str^[OutStr.len] := 0C;

END dStr0Ins;

(*==================================================================*)
 PROCEDURE dStrRepl(InStr:DynStr; Start:INTEGER; VAR OutStr:DynStr);
(*==================================================================*)
    (* Copies source into destination, starting at position startPos. Copying stops when
       all of source has been copied, or when it reaches the siz of the destination.     *)
    (* If Start pos is beyond end of InStr, chars are appended at the end                *)

VAR  NumOffEnd,
     NumCopied :INTEGER;
     i, k   :INTEGER;

BEGIN

IF Start < 0 THEN
   Start := 0;
ELSIF Start >= VAL(INTEGER, OutStr.len) THEN
   Start := OutStr.len;
END;

NumCopied := InStr.len;
NumOffEnd := mini(NumCopied-(VAL(INTEGER, OutStr.siz)-Start), NumCopied);
(*DebugPause("NumOffEnd", NumOffEnd);*)
IF NumOffEnd > 0 THEN
   DEC(NumCopied, NumOffEnd);
END;

k := Start;
i := 0;
FOR i := 0 TO NumCopied-1 DO
   OutStr.str^[k] := InStr.str^[i];
   INC(k);
END;

OutStr.len := maxi(OutStr.len, Start+NumCopied);
OutStr.str^[OutStr.len] := 0C;

END dStrRepl;

(*==========================================================================*)
 PROCEDURE dStr0Repl(InStr:ARRAY OF CHAR; Start:INTEGER; VAR OutStr:DynStr);
(*==========================================================================*)

VAR  NumOffEnd,
     NumCopied :INTEGER;
     i, k   :INTEGER;

BEGIN

IF Start < 0 THEN
   Start := 0;
ELSIF Start >= VAL(INTEGER, OutStr.len) THEN
   Start := OutStr.len;
END;

NumCopied := LENGTH(InStr);
NumOffEnd := mini(NumCopied-(VAL(INTEGER, OutStr.siz)-Start), NumCopied);
(*DebugPause("NumOffEnd", NumOffEnd);*)
IF NumOffEnd > 0 THEN
   DEC(NumCopied, NumOffEnd);
END;

k := Start;
i := 0;
FOR i := 0 TO NumCopied-1 DO
   OutStr.str^[k] := InStr[i];
   INC(k);
END;

OutStr.len := maxi(OutStr.len, Start+NumCopied);
OutStr.str^[OutStr.len] := 0C;

END dStr0Repl;


END DynStr1b.

