
(*##############################*)
 IMPLEMENTATION MODULE DynStr1NC;           (* vx.x-100402 *)
(*##############################*)
                                       (* by Thomas Breeden, tmb@virginia.edu *)
(*
    Dynamically allocated string processing: Common comparison procedures,
                                             case insensitive
*)

(*
Copyright (C) 2002 Thomas Breeden.

Permission to use, copy, modify and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appear in all copies and that both the copyright
notice and this permission notice appear in supporting documentation.
Thomas Breeden makes no representations about the suitability of this software
for any purpose. It is provided "as is" without express or implied warranty.
*)

FROM DynStr0 IMPORT DynStr, dStrInC, dStrInE, dStrCreate, dStrDispose;

VAR  dstemp :DynStr;

(* VERSION HISTORY

vx.x-100402

*)

(*---------------------------------------*)
 PROCEDURE Minimum(A,B:CARDINAL):CARDINAL;
(*---------------------------------------*)
BEGIN

IF A < B THEN RETURN A ELSE RETURN B END;

END Minimum;

(*======================================*)
 PROCEDURE dStrCmpNC(s1,s2:DynStr):INTEGER;
(*======================================*)
      (* Compare two strings *)
      (* Returns 0 for equality, -1 if S1 < S2, +1 if S1 > S2 *)

(*

OUTPUT
     <function value>    +1 = S1 > S2
                          0 = equality
                         -1 = S1 < S2
*)

VAR  MinLen,
     i          :CARDINAL;

BEGIN WITH s1 DO

MinLen := Minimum(len, s2.len);

i := 0;
LOOP
   IF (i = MinLen) OR (CAP(str^[i]) # CAP(s2.str^[i])) THEN
      EXIT;
   END;
   INC(i);
END;

IF i = MinLen THEN
   IF len < s2.len THEN
      RETURN -1
   ELSIF len > s2.len THEN
      RETURN 1
   ELSE
      RETURN 0
   END;
ELSE
   IF CAP(str^[i]) > CAP(s2.str^[i]) THEN
      RETURN 1
   ELSE
      RETURN -1
   END;
END;

END END dStrCmpNC;

(*=======================================================*)
 PROCEDURE dStr0CmpNC(s1:ARRAY OF CHAR; s2:DynStr):INTEGER;
(*=======================================================*)

VAR  iRes :INTEGER;

BEGIN

IF dstemp.siz < LENGTH(s1) THEN
   dStrDispose(dstemp);
   dStrInC(s1, dstemp);
ELSE
   dStrInE(s1, dstemp);
END;

iRes := dStrCmpNC(dstemp, s2);

IF dstemp.siz > 160 THEN
   dStrDispose(dstemp);
   dStrCreate(dstemp, 20);
END;

RETURN iRes;

END dStr0CmpNC;

(*=========================================================*)
 PROCEDURE dStrPosNC(s,Pattern:DynStr; Start:INTEGER):INTEGER;
(*=========================================================*)
      (* Find the position of a substring in a string *)
      (* Start looking at position Start *)
      (* Returns < 0 if not a substring *)

(*
NOTE
        1.Nothing is ever found within the nullstr.
        2.Nothing is ever found if Start is beyond the end of the
          searched string.
        3.Otherwise, the nullstr is always found as a Pattern at Start.
*)

VAR  sLen,
     PatternLen,
     SearchLimit,
     PatternLimit,
     i,j                :INTEGER;
     Equal              :BOOLEAN;

BEGIN WITH s DO

IF Start < 0 THEN
   Start := 0
END;

PatternLen := Pattern.len;
PatternLimit := PatternLen-1;
sLen := len;
SearchLimit :=  sLen - PatternLen;

IF (Start+PatternLen) > sLen THEN
   RETURN -1
ELSIF PatternLen = 0 THEN
   IF Start < sLen THEN
      RETURN Start
   ELSE RETURN -1
   END;
ELSE
   i := Start-1;
   REPEAT
      INC(i);
      j := -1;
      REPEAT
        INC(j);
        Equal := (CAP(str^[i+j]) = CAP(Pattern.str^[j]));
      UNTIL NOT Equal OR (j = PatternLimit);
   UNTIL Equal OR (i = SearchLimit);
   IF NOT Equal THEN
      RETURN -1
   ELSE RETURN i
   END;
END;

END END dStrPosNC;

(*===========================================================================*)
 PROCEDURE dStr0PosNC(s:DynStr; Pattern:ARRAY OF CHAR; Start:INTEGER) :INTEGER;
(*===========================================================================*)
     (* Find the position of a "Str0" fromat substring in a DynStr *)

VAR  iRes :INTEGER;

BEGIN

IF dstemp.siz < LENGTH(Pattern) THEN
   dStrDispose(dstemp);
   dStrInC(Pattern, dstemp);
ELSE
   dStrInE(Pattern, dstemp);
END;

iRes := dStrPosNC(s, dstemp, Start);

IF dstemp.siz > 160 THEN
   dStrDispose(dstemp);
   dStrCreate(dstemp, 20);
END;

RETURN iRes;

END dStr0PosNC;

(*===========================================================================*)
 PROCEDURE dStrPos0NC(s:ARRAY OF CHAR; Pattern:DynStr; Start:INTEGER) :INTEGER;
(*===========================================================================*)
     (* Find the position of a Dynstr in a "Str0" format string *)

VAR  iRes :INTEGER;

BEGIN

IF dstemp.siz < LENGTH(s) THEN
   dStrDispose(dstemp);
   dStrInC(s, dstemp);
ELSE
   dStrInE(s, dstemp);
END;

iRes := dStrPosNC(dstemp, Pattern, Start);

IF dstemp.siz > 160 THEN
   dStrDispose(dstemp);
   dStrCreate(dstemp, 20);
END;

RETURN iRes;

END dStrPos0NC;

(*-------------------------*)
 BEGIN (* module init code *)
(*-------------------------*)

dStrCreate(dstemp, 20);

END DynStr1NC.
