
(*#############################*)
 IMPLEMENTATION MODULE DynStr2A;         (* v0.1-040404 *)
(*#############################*)
                                       (* by Thomas Breeden, tmb@virginia.edu *)
(*
   See other string approaches in modules: Str0, Str1, ...
                                           DynStr0, DynStr1, ...
                                           PasStr0, PasStr1, ...
*)

(*
Copyright (C) 1993-2004 Thomas Breeden.

Permission to use, copy, modify and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appear in all copies and that both the copyright
notice and this permission notice appear in supporting documentation.
Thomas Breeden makes no representations about the suitability of this software
for any purpose. It is provided "as is" without express or implied warranty.
*)

FROM DynStr0 IMPORT DynStr;
FROM DynStr1 IMPORT dStrSeg;
IMPORT DynStr3 (*dStrSeg*);

FROM Str1    IMPORT ChrAsg, ChrCat, ChrPos;

(* NOTE:
   0. dStrLeading and dStrCollapse will not change the .siz of the passed
      DynStr.

      The "token" parameter output from dNextToken(), HOWEVER, will be
      dynamically reallocated to be bigger (or smaller) as handled by
      the DynStr3 module.

VERSION HISTORY

vx.x-071093   -First

    -060998   -removed "VAR" for delims parameter.

    -111699   -Fixed behavior of StrCollapse() to agree with that of Str2 (and its
                    documentation). Now it replaces
               all delimiters and delimiter runs in the input file with
                                        the character delims[0].
                                        Specifically for WhiteSpace, it can be used to change tabs in the
                                        input string to spaces.

v0.1-122801   -Fix in dStrLeading for index exception if sent a 0 length string.
    -040404   -Added proc dCollapseLeading()
*)

(*----------------------------------------------------------------------------------------*)
 PROCEDURE LeadingProcess(VAR s:DynStr; delims:ARRAY OF CHAR; ch:CHAR; doCollapse:BOOLEAN);
(*----------------------------------------------------------------------------------------*)

VAR i         :INTEGER;
    leftjust  :BOOLEAN;
    chRepl    :CHAR;

BEGIN

IF ch = 0C THEN
   chRepl := ' ';
ELSE
   chRepl := ch;
END;
leftjust := doCollapse;

i := 0;
WHILE (i < VAL(INTEGER, s.len)) AND (ChrPos(delims, s.str^[i], 0) >= 0) DO
   s.str^[i] := chRepl;
   INC(i);
END;

IF leftjust THEN

   IF ch # 0C THEN
      DEC(i);
   END;

   dStrSeg(s, i, VAL(INTEGER, s.len)-1, s);

END;

END LeadingProcess;

(*=================================================================*)
 PROCEDURE dStrLeading(VAR s:DynStr; delims:ARRAY OF CHAR; ch:CHAR);
(*=================================================================*)

BEGIN

LeadingProcess(s, delims, ch, ch = 0C);

END dStrLeading;

(*=================================================================*)
 PROCEDURE dStrCollapseLeading(VAR s:DynStr; delims:ARRAY OF CHAR);
(*=================================================================*)
BEGIN

LeadingProcess(s, delims, delims[0], TRUE);

END dStrCollapseLeading;

(*=============================================================*)
 PROCEDURE dStrCollapse(VAR s:DynStr; delims:ARRAY OF CHAR);
(*=============================================================*)

(* Collapse all runs of embedded delims into one delim *)

VAR  i,j  :INTEGER;
     ch,
     last :CHAR;

BEGIN

last := 0C;
j := 0;

FOR i := 0 TO VAL(INTEGER, s.len)-1 DO

   ch := s.str^[i];

   IF ChrPos(delims, ch, 0) >= 0 THEN
      IF last # delims[0] THEN
         s.str^[j] := delims[0];
         INC(j);
         last := delims[0];
      END;

   ELSE
      s.str^[j] := s.str^[i];
      INC(j);
      last := ch;

   END;
END;

s.len := j;

END dStrCollapse;

(*============================================================*)
 PROCEDURE dSkipDelims(VAR s:DynStr; delims:ARRAY OF CHAR;
                       VAR index:INTEGER);
(*============================================================*)

BEGIN

WHILE (index < VAL(INTEGER, s.len)) AND (ChrPos(delims, s.str^[index], 0) >= 0) DO
   INC(index);
END;

END dSkipDelims;

(*===============================================================*)
 PROCEDURE dSkipNonDelims(VAR s:DynStr; delims:ARRAY OF CHAR;
                          VAR index:INTEGER);
(*===============================================================*)

BEGIN

WHILE (index < VAL(INTEGER, s.len)) AND (ChrPos(delims, s.str^[index], 0) < 0) DO
   INC(index);
END;

END dSkipNonDelims;

(*============================================================*)
 PROCEDURE dNextToken(VAR s:DynStr; delims:ARRAY OF CHAR;
                      VAR token:DynStr; VAR index:INTEGER);
(*============================================================*)

VAR  start  :INTEGER;

BEGIN

dSkipDelims(s, delims, index);
start := index;

dSkipNonDelims(s, delims, index);
DynStr3.dStrSeg(s, start, index-1, token);

END dNextToken;

(*--------------------*)
 BEGIN (* module init *)
(*--------------------*)

ChrAsg(' ', WhiteSpace);
ChrCat(11C, WhiteSpace);

END DynStr2A.
