
(*###############################*)
 IMPLEMENTATION MODULE TextIOHelper;            (* v0.1-040504 *)
(*###############################*)
                                     (* by Thomas Breeden, tmb@virginia.edu *)
(*
   Procedures to help with the use of the ISO standard TextIO input routines.

       > Handles the calls to SkipLine()
       > Reads the last line in a file correctly whether it is followed by an
         endofline or not.
       > Supports optional silent discarding of empty lines.
       > Supports reading lines of arbitrary length.
*)

(*
Copyright (C) 2003-2004 Thomas Breeden.

Permission to use, copy, modify and distribute this software and its
documentation for any purpose and without fee is hereby granted, provided that
the above copyright notice appear in all copies and that both the copyright
notice and this permission notice appear in supporting documentation.
Thomas Breeden makes no representations about the suitability of this software
for any purpose. It is provided "as is" without express or implied warranty.
*)

FROM IOChan       IMPORT ChanId;
FROM IOConsts     IMPORT ReadResults;
FROM IOResult     IMPORT ReadResult;
FROM TextIO       IMPORT ReadRestLine, ReadString, SkipLine;

FROM DynStr0  IMPORT DynStr;
IMPORT DynStr0, DynStr1, DynStr3, Str0, Str1;

(* VERSION HISTORY

vx.x-111403  - First

v0.1-040504  - added NumSkipped parameter to xReadNextNonBlankLine() so that file line numbers can be
               tracked accurately.
*)

(*=========================================================================*)
 PROCEDURE ReadNextLine(cid:ChanId; VAR OneLine:ARRAY OF CHAR):ReadResults;
(*=========================================================================*)

VAR  rRes  :ReadResults;

BEGIN

ReadRestLine(cid, OneLine);
rRes := ReadResult(cid);

IF rRes = endOfLine THEN
   Str1.StrClr(OneLine);
END;

IF rRes # endOfInput THEN
   rRes := allRight;
   SkipLine(cid);
END;

RETURN rRes;

END ReadNextLine;

(*=========================================================================================================*)
 PROCEDURE ReadNextNonBlankLine(cid:ChanId; VAR OneLine:ARRAY OF CHAR; VAR NumSkipped:CARDINAL):ReadResults;
(*=========================================================================================================*)

VAR  len   :CARDINAL;
     rRes  :ReadResults;

BEGIN

NumSkipped := 0;

REPEAT

   rRes := ReadNextLine(cid, OneLine);
   len := LENGTH(OneLine);

   IF (len = 0) AND (rRes # endOfInput) THEN
      INC(NumSkipped);
   END;

UNTIL (len > 0) OR (rRes = endOfInput);

RETURN rRes;

END ReadNextNonBlankLine;

(*====================================================================*)
 PROCEDURE dReadNextLine(cid:ChanId; VAR dOneLine:DynStr):ReadResults;
(*====================================================================*)

VAR  buf   :Str0.String80;
     rRes  :ReadResults;

BEGIN

IF dOneLine.str = NIL THEN
   DynStr0.dStrCreate(dOneLine, 80);
ELSE
   DynStr1.dStrClr(dOneLine);
END;

ReadString(cid, buf);
rRes := ReadResult(cid);

WHILE (rRes # endOfLine) AND (rRes # endOfInput) DO
   DynStr3.dStr0Cat(buf, dOneLine);
   ReadString(cid, buf);
   rRes := ReadResult(cid);
END;

IF rRes = endOfLine THEN
   rRes := allRight;
   SkipLine(cid);
END;

RETURN rRes;

END dReadNextLine;

(*=====================================================================================================*)
 PROCEDURE dReadNextNonBlankLine(cid:ChanId; VAR dOneLine:DynStr; VAR NumSkipped:CARDINAL):ReadResults;
(*=====================================================================================================*)

VAR  rRes  :ReadResults;

BEGIN

NumSkipped := 0;

REPEAT

   rRes := dReadNextLine(cid, dOneLine);

   IF (dOneLine.len = 0) AND (rRes # endOfInput) THEN
      INC(NumSkipped);
   END;

UNTIL (dOneLine.len > 0) OR (rRes = endOfInput);

RETURN rRes;

END dReadNextNonBlankLine;

END TextIOHelper.
