(*###########*)
 MODULE tgM2;       (* $VER: tgM2.mod 0.9 (19.5.4) *)
(*###########*)
(*
   This file is part of 'tgM2'.

   'tgM2' : A Test Driver Generator for Modula-2 programs
   Copyright (C) 2004 by Tom Breeden.

   'tgM2' is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   'tgM2' is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'tgM2'; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*)
(*
   'tgM2' was patterned on the Ada 95 program, 'tg' which is copyright 1996
    by Andre Spiegel with modifications by Ralf Reissing.
*)

FROM ChanConsts   IMPORT ChanFlags, FlagSet, OpenResults;
FROM IOChan       IMPORT ChanId, InvalidChan;
FROM SeqFile      IMPORT Close, OpenRead, OpenWrite;
FROM StdChans     IMPORT SetInChan, SetOutChan, StdErrChan,
                         StdInChan, StdOutChan;
FROM STextIO      IMPORT WriteLn, WriteString;

IMPORT tgM2Version;
FROM tgM2TranslatorISO IMPORT Output_Mode, Translate;

FROM tgM2Exceptions    IMPORT IstgException, tgException, tgExceptions, Raise;

FROM ArgsSupport       IMPORT DisposeParamEntry, GetParams, InitParamEntry, ParamsArrayEntry,
                              ParamsTypes, ProcessCLI;
FROM DynStr0           IMPORT DynStr;

IMPORT DynStr0, DynStr1, DynStr3, Str0, Str1;

(* TO DO:
     1. Program result code
     2. need to be able to Dispose of structure in order to run on myself.
*)

CONST PgmTemplate = "-f=-fail/K,-p=-pass/K,-script/A,-driver,-overwrite/S,-h=-help/S";
TYPE  PgmCmds = (FailModeCmd, PassModeCmd, ScriptFileCmd, DriverFileCmd, OverwriteCmd, HelpCmd);

CONST OutputModeTemplate = "off/S,numbers/S,titles/S,full/S";
(* Translator.Output_Mode is (Off, Numbers, Titles, Full) *)

VAR  Script_Name,
     Driver_Name     :DynStr;
     Script_File,
     Driver_File     :ChanId;
     OverwriteDriver :BOOLEAN;

     Fail_Output,
     Pass_Output     :Output_Mode;

(*----------------*)
 PROCEDURE InitPgm;
(*----------------*)
BEGIN

Script_File := InvalidChan();
Driver_File := InvalidChan();

END InitPgm;

(*----------------------------------------*)
 PROCEDURE WriteStringLn(s:ARRAY OF CHAR);
(*----------------------------------------*)

BEGIN

WriteString(s); WriteLn;

END WriteStringLn;

(*-------------------*)
 PROCEDURE Put_Usage;
(*-------------------*)

BEGIN

WriteString('tgM2: A Test Driver Generator for Modula-2 programs ');
WriteString(tgM2Version.Id); WriteStringLn(').');
WriteLn;

WriteStringLn('Usage:   tgM2 [options...] script_file [driver_file]');
WriteLn;
WriteStringLn('Options:');
WriteStringLn('-f <setting>   Specifies how failed test cases will be reported');
WriteStringLn('               by the testdriver. <setting> may be one of');
WriteLn;
WriteStringLn('               off      No output.');
WriteStringLn('               numbers  Only test case numbers, followed by the string');
WriteStringLn('                        "FAIL.".');
WriteStringLn('               titles   Numbers and titles, followed by "...FAIL."');
WriteStringLn('                        on the next line.');
WriteStringLn('               full     Numbers and titles, "...FAIL" and a short');
WriteStringLn('                        explanation on the next line.');
WriteStringLn('                        This is the default.');
WriteLn;
WriteStringLn('-p <setting>   Same as "-f", but for passed test cases.');
WriteStringLn('               Default is "numbers".');
WriteLn;
WriteStringLn('-overwrite     Overwrite any existing driver file of the same name');
WriteLn;
WriteStringLn('PgmTemplate is "-f=-fail/K,-p=-pass/K,-script/A,-driver,-overwrite/S,-h=-help/S"');
WriteLn;

END Put_Usage;

(*----------------------------------------------------------------------------------------*)
 PROCEDURE Get_Arguments(VAR Fail_Output, Pass_Output:Output_Mode;
                         VAR Script_Name, Driver_Name:DynStr; VAR OverwriteDriver:BOOLEAN);
(*----------------------------------------------------------------------------------------*)

VAR  Params        :ARRAY PgmCmds OF ParamsArrayEntry;
     cmd           :PgmCmds;
     OutModeParams :ARRAY Output_Mode OF ParamsArrayEntry;
     outmode       :Output_Mode;
     dTempl        :DynStr;
     bRes          :BOOLEAN;

BEGIN

InitParamEntry(StrParam, Params[FailModeCmd]);
InitParamEntry(StrParam, Params[PassModeCmd]);
InitParamEntry(StrParam, Params[ScriptFileCmd]);
InitParamEntry(StrParam, Params[DriverFileCmd]);
InitParamEntry(SwitchParam, Params[OverwriteCmd]);
InitParamEntry(SwitchParam, Params[HelpCmd]);

InitParamEntry(SwitchParam, OutModeParams[Off]);
InitParamEntry(SwitchParam, OutModeParams[Numbers]);
InitParamEntry(SwitchParam, OutModeParams[Titles]);
InitParamEntry(SwitchParam, OutModeParams[Full]);

DynStr0.dStrInC(OutputModeTemplate, dTempl);

Fail_Output := Full;
Pass_Output := Numbers;

bRes := ProcessCLI(PgmTemplate, Params);

IF NOT bRes OR Params[HelpCmd].Given THEN
   Raise(Incorrect_Call);
END;

DynStr0.dStrAsgC(Params[ScriptFileCmd].Val.vs, Script_Name);

IF Params[DriverFileCmd].Given THEN
   DynStr0.dStrAsgC(Params[DriverFileCmd].Val.vs, Driver_Name);
ELSE
   DynStr0.dStrCreate(Driver_Name, 2);
END;

IF Params[FailModeCmd].Given THEN

   bRes := GetParams(Params[FailModeCmd].Val.vs, dTempl, OutModeParams);
   IF NOT bRes THEN
      Raise(Incorrect_Call);
   END;

   FOR outmode := MIN(Output_Mode) TO MAX(Output_Mode) DO
      IF OutModeParams[outmode].Given THEN
         Fail_Output := outmode;
      END;
   END;

END;

IF Params[PassModeCmd].Given THEN

   bRes := GetParams(Params[PassModeCmd].Val.vs, dTempl, OutModeParams);
   IF NOT bRes THEN
      Raise(Incorrect_Call);
   END;

   FOR outmode := MIN(Output_Mode) TO MAX(Output_Mode) DO
      IF OutModeParams[outmode].Given THEN
         Pass_Output := outmode;
      END;
   END;

END;

OverwriteDriver := Params[OverwriteCmd].Given;

FOR outmode := MIN(Output_Mode) TO MAX(Output_Mode) DO
   DisposeParamEntry(OutModeParams[outmode]);
END;

FOR cmd := MIN(PgmCmds) TO MAX(PgmCmds) DO
   DisposeParamEntry(Params[cmd]);
END;

DynStr0.dStrDispose(dTempl);

END Get_Arguments;

(*----------------------------------------------------------------------------------------------------------------*)
 PROCEDURE tg_main(Script_Name, Driver_Name:DynStr; Fail_Output, Pass_Output:Output_Mode; OverwriteDriver:BOOLEAN);
(*----------------------------------------------------------------------------------------------------------------*)

VAR ModuleName,
    Output_Name   :DynStr;
    overwritespec :FlagSet;
    oRes          :OpenResults;

            (*----------------------------------------------------------------*)
             PROCEDURE Without_Extension(File_Name:DynStr; VAR FNwoExt:DynStr);
            (*----------------------------------------------------------------*)

            VAR  Index :CARDINAL;
                 found :BOOLEAN;

            BEGIN

            IF FNwoExt.str = NIL THEN
               DynStr0.dStrAsgC(File_Name, FNwoExt);
            ELSE
               DynStr3.dStrAsgE(File_Name, FNwoExt);
            END;

            Index := File_Name.len-1;
            found := FALSE;
            WHILE NOT found AND (Index > 0) DO
              IF File_Name.str^[Index] = '.' THEN
                DynStr1.dStrSeg(File_Name, 0, Index-1, FNwoExt);
                found := TRUE;
              ELSE
                DEC(Index);
              END;
            END;

            END Without_Extension;

            (*----------------------------------------------------------------------*)
             PROCEDURE Without_ExtensionOrPath(File_Name:DynStr; VAR FNwoPth:DynStr);
            (*----------------------------------------------------------------------*)

            VAR  Index :CARDINAL;
                 found :BOOLEAN;

            BEGIN

            Without_Extension(File_Name, FNwoPth);

            Index := FNwoPth.len-1;
            found := FALSE;
            WHILE NOT found AND (Index > 0) DO
              IF (FNwoPth.str^[Index] = '\') OR (FNwoPth.str^[Index] = '/')
                     OR (FNwoPth.str^[Index] = ':') THEN
                DynStr1.dStrSeg(FNwoPth, Index+1, FNwoPth.len-1, FNwoPth);
                found := TRUE;
              ELSE
                DEC(Index);
              END;
            END;

            END Without_ExtensionOrPath;

BEGIN

ModuleName.str := NIL;
Output_Name.str := NIL;

(* attempt to open the input file, error if it doesn't exist *)
OpenRead(Script_File, Script_Name.str^, FlagSet{}, oRes);
IF oRes # opened THEN
   IF oRes = noSuchFile THEN
      WriteString("tgM2: File "); WriteString(Script_Name.str^); WriteStringLn(" not found.");
   ELSE
      WriteString("tgM2: Error when attempting to open "); WriteStringLn(Script_Name.str^);
   END;
   HALT (*Raise(Terminate_Program)*);
END;

(* create the output file, overwrite if it does exist *)
IF Driver_Name.len = 0 THEN
   Without_Extension(Script_Name, Output_Name);
   DynStr3.dStr0Cat(".mod", Output_Name);
ELSE
   DynStr0.dStrAsgC(Driver_Name, Output_Name);
END;

IF OverwriteDriver THEN
   overwritespec := FlagSet{oldFlag};
ELSE
   overwritespec := FlagSet{};
END;

OpenWrite(Driver_File, Output_Name.str^, overwritespec, oRes);
IF oRes # opened THEN
   WriteString("tgM2: Error creating file "); WriteString(Output_Name.str^);
   IF oRes = fileExists THEN
      WriteString(" - file exists");
   END;
   WriteLn;
   HALT (*Raise(Terminate_Program)*);
END;

Without_ExtensionOrPath(Output_Name, ModuleName);

SetOutChan(Driver_File);
SetInChan(Script_File);

Translate(Script_File, Driver_File, StdErrChan(), ModuleName,
          Fail_Output, Pass_Output);

END tg_main;

(*--------------*)
 BEGIN  (* tgM2 *)
(*--------------*)

InitPgm;

Get_Arguments(Fail_Output, Pass_Output, Script_Name, Driver_Name, OverwriteDriver);

tg_main(Script_Name, Driver_Name, Fail_Output, Pass_Output, OverwriteDriver);

(*-----*)
 EXCEPT
(*-----*)

IF IstgException() THEN
   IF tgException() = Incorrect_Call THEN
      Put_Usage;
      RETURN;
   ELSIF tgException() = File_Error THEN
      WriteString("tgM2: File_Error raised ");
      WriteLn();
   END;
END;

(*------*)
 FINALLY
(*------*)

IF (Driver_File # InvalidChan()) AND (Driver_File # StdOutChan()) THEN
   SetOutChan(StdOutChan());
   Close(Driver_File);
END;

IF (Script_File # InvalidChan()) AND (Script_File # StdInChan()) THEN
   SetInChan(StdInChan());
   Close(Script_File);
END;

END tgM2.
