
(*###############################*)
 IMPLEMENTATION MODULE tgM2Parser;           (* $VER: tgM2Parser.mod 0.9 (19.5.04) *)
(*###############################*)
(*
   This file is part of 'tgM2'.

   'tgM2' : A Test Driver Generator for Modula-2 programs
   Copyright (C) 2004 by Tom Breeden.

   'tgM2' is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   'tgM2' is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'tgM2'; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*)
(*
   'tgM2' was patterned on the Ada 95 program, 'tg' which is copyright 1996
    by Andre Spiegel with modifications by Ralf Reissing.
*)

FROM SYSTEM              IMPORT ADDRESS;
FROM Storage             IMPORT ALLOCATE;
FROM Debugging0          IMPORT Assert;

FROM WholeStr            IMPORT StrToCard;
FROM ConvTypes           IMPORT ConvResults;

FROM tgM2Scanner         IMPORT Current_Token, Next_Token, Token, Token_Text;
FROM tgM2Exceptions      IMPORT Raise, tgExceptions;

FROM DynStr0             IMPORT DynStr;
FROM ListV2Processor     IMPORT ListAddition, ListCreate, ListOpDone, NilList;
FROM TextBlocks          IMPORT AllocBlock, AppendBlockdStr, InvalidBlock,
                                NewBlockLine;
IMPORT Str0, Str1, DynStr0;

CONST DEBUG = FALSE;

(*-------------------------------------------------*)
 PROCEDURE NEWTestCase(VAR TC:Test_Case_Type);
(*-------------------------------------------------*)

BEGIN

NEW(TC);
Assert(TC # NIL, "Out of memory in NEWTestCase");
WITH TC^ DO
   Number := 0;
   DynStr0.dStrCreate(Title, 32);
   ExceptionRequired := FALSE;
   Define_Part  := InvalidBlock();
   Prepare_Part := InvalidBlock();
   Test_Part    := InvalidBlock();
   Pass_Part := NilList();
   Cleanup_Part := InvalidBlock();
END;

END NEWTestCase;

(*-------------------------------------------------*)
 PROCEDURE NEWGeneralPart(VAR GP:General_Part_Type);
(*-------------------------------------------------*)

BEGIN

NEW(GP);
Assert(GP # NIL, "Out of memory in NEWGeneralPart");
WITH GP^ DO
   Fail_Handling  := Continue;
   Error_Handling := Stop;
   Context        := InvalidBlock();
   (*Exceptions     := InvalidBlock();*)
   Define_Part    := InvalidBlock();
END;

END NEWGeneralPart;

(*------------------------------------------------------------------------*)
 PROCEDURE Get_Code_Lines(VAR Data:(*in out*)Code_List; VAR Empty:BOOLEAN);
(*------------------------------------------------------------------------*)

VAR  tt   :DynStr;

BEGIN

IF Current_Token() # Indented_Text_Line THEN
   Raise(Parse_Error);
END;

Empty := TRUE;
tt.str := NIL;

LOOP
   Token_Text(tt);
   IF tt.len > 0 THEN
      Empty := FALSE;
   END;
   AppendBlockdStr(Data, tt);
   NewBlockLine(Data);
   Next_Token;
   IF Current_Token() # Indented_Text_Line THEN
      EXIT;
   END;
END;

DynStr0.dStrDispose(tt);

END Get_Code_Lines;

(*-----------------------------------------------------------------*)
 PROCEDURE Get_Test_Case_Title(Test_Case:(*in out*)Test_Case_Type);
(*-----------------------------------------------------------------*)

VAR  numstr :DynStr;
     cvRes  :ConvResults;

BEGIN

numstr.str := NIL;

Next_Token;
IF Current_Token() = Test_Case_Number THEN

   Token_Text(numstr);
   StrToCard(numstr.str^, Test_Case^.Number, cvRes);
   DynStr0.dStrDispose(numstr);
   IF cvRes # strAllRight THEN
      (*exception*)
      Test_Case^.Number := 0;
      Next_Token;
      Token_Text(Test_Case^.Title);
      RETURN;
   END;

   Next_Token;

   IF Current_Token() = Test_Case_Title THEN
      Token_Text(Test_Case^.Title);
   ELSE
      Raise(Parse_Error);
   END;

ELSIF Current_Token() = Test_Case_Title THEN

   Test_Case^.Number := 0;
   Token_Text(Test_Case^.Title);

ELSE

   Raise(Parse_Error);

END;

END Get_Test_Case_Title;

(*-----------------------------------------------------------*)
 PROCEDURE Get_Pass_Clause(Data:(*in out*)Pass_Clause_Type);
(*-----------------------------------------------------------*)

VAR  EmptyClause   :BOOLEAN;

BEGIN

Data^.Predicate := AllocBlock();

IF Current_Token() = Key_Exception THEN
   Data^.Path := ExceptionPath;
   Next_Token;
ELSIF Current_Token() = Indented_Text_Line THEN
   Data^.Path := NormalPath;
ELSE
   Raise(Parse_Error);
END;

Get_Code_Lines(Data^.Predicate, EmptyClause);

IF EmptyClause THEN
   Raise(Parse_Error);
END;

END Get_Pass_Clause;

(*==============================================================*)
 PROCEDURE Get_General_Part(VAR Data:(*in out*)General_Part_Type);
(*==============================================================*)

VAR  Empty :BOOLEAN;

BEGIN

Assert(Data = NIL, "Param Data must be NIL on input to Get_General_Part");
NEWGeneralPart(Data);

IF Current_Token() = Key_Fail_Handling THEN
   Next_Token;
   CASE Current_Token() OF
      Key_Stop:
         Data^.Fail_Handling := Stop;
         |
      Key_Continue:
         Data^.Fail_Handling := Continue;
         |
      ELSE
         Raise(Parse_Error);
   END;
   Next_Token;
END;

IF Current_Token() = Key_Error_Handling THEN
   Next_Token;
   CASE Current_Token() OF
      Key_Stop:
         Data^.Error_Handling := Stop;
         |
      Key_Continue:
         Data^.Error_Handling := Continue;
         |
      ELSE
         Raise(Parse_Error);
   END;
   Next_Token;
END;

IF Current_Token() = Key_Context THEN
   Next_Token;
   Data^.Context := AllocBlock();
   Get_Code_Lines(Data^.Context, Empty);
END;

IF Current_Token() = Key_Define THEN
   Next_Token;
   Data^.Define_Part := AllocBlock();
   Get_Code_Lines(Data^.Define_Part, Empty);
END;

END Get_General_Part;

(*========================================================*)
 PROCEDURE Get_Code_Part(VAR Data:(*in out*)Code_List);
(*========================================================*)

VAR  Empty :BOOLEAN;

BEGIN

Assert(Data = InvalidBlock(), "Param Data must be InvalidBlock() on input to Get_Code_Part");

Data :=  AllocBlock();

IF Current_Token() = Key_Code THEN
   Next_Token;
   Get_Code_Lines(Data, Empty);
END;

END Get_Code_Part;

(*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*)
 PROCEDURE SeqOrdering(A, B:ADDRESS):INTEGER;
(*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*)

BEGIN

IF A = B THEN
   RETURN 0;
ELSE
   RETURN -1;
END;

END SeqOrdering;

(*=========================================================*)
 PROCEDURE Get_Test_Case(VAR Data:(*in out*)Test_Case_Type);
(*=========================================================*)

VAR  Pass_Clause :Pass_Clause_Type;
     Empty       :BOOLEAN;

BEGIN

Assert(Data = NIL, "Param Data must be NIL on input to Get_Test_Case");
NEWTestCase(Data);

IF Current_Token() = Key_Begin_Test_Case THEN

   Get_Test_Case_Title(Data);
   Next_Token;

   IF Current_Token() = Key_Define THEN
      Next_Token;
      Data^.Define_Part := AllocBlock();
      Get_Code_Lines(Data^.Define_Part, Empty);
   END;

   IF Current_Token() = Key_Prepare THEN
      Next_Token;
      Data^.Prepare_Part := AllocBlock();
      Get_Code_Lines(Data^.Prepare_Part, Empty);
   END;

   IF Current_Token() = Key_Test THEN
      Next_Token;
      Data^.Test_Part := AllocBlock();
      Get_Code_Lines(Data^.Test_Part, Empty);
   ELSE
      Raise(Parse_Error);
   END;

   IF Current_Token() # Key_Pass THEN
      Raise(Parse_Error);
   ELSE
      ListCreate(Data^.Pass_Part, SeqOrdering, FALSE(*Duplicates*));
      Assert(ListOpDone(Data^.Pass_Part), "ListCreate FAIL");
      WHILE Current_Token() = Key_Pass DO

         Next_Token;

         NEW(Pass_Clause);
         (*Pass_Clause^.Path.str := NIL;*)

         Get_Pass_Clause(Pass_Clause);
         IF Pass_Clause^.Path = ExceptionPath THEN
            Data^.ExceptionRequired := TRUE;
         END;

         ListAddition(Data^.Pass_Part, Pass_Clause);
         Assert(ListOpDone(Data^.Pass_Part), "ListAddition FAIL");

      END;
   END;

   IF Current_Token() = Key_Cleanup THEN
      Next_Token;
      Data^.Cleanup_Part := AllocBlock();
      Get_Code_Lines(Data^.Cleanup_Part, Empty);
   END;

END;

END Get_Test_Case;

END tgM2Parser.


