
(*#################################*)
 IMPLEMENTATION MODULE tgM2Scanner;        (* $VER: tgM2Scanner.mod 0.9 (19.5.04) *)
(*#################################*)
(*
   This file is part of 'tgM2'.

   'tgM2' : A Test Driver Generator for Modula-2 programs
   Copyright (C) 2004 by Tom Breeden.

   'tgM2' is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   'tgM2' is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'tgM2'; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*)
(*
   'tgM2' was patterned on the Ada 95 program, 'tg' which is copyright 1996
    by Andre Spiegel with modifications by Ralf Reissing.
*)

FROM SYSTEM       IMPORT ADR;

FROM IOChan       IMPORT ChanId, InvalidChan;
FROM IOConsts     IMPORT ReadResults;
FROM SeqFile      IMPORT IsSeqFile, Reread;
FROM WholeStr     IMPORT CardToStr;

FROM DynStr0       IMPORT DynStr;
FROM TextIOHelper  IMPORT dReadNextNonBlankLine;

IMPORT DynStr0, DynStr1, DynStr1NC, DynStr2, DynStr2A, DynStr3, Str0, Str1;

VAR  OneLine,
     dToken   :DynStr;
     linePos  :INTEGER;
     scanfil  :ChanId;
     curTok,
     prevTok  :Token;
     curLine,
     TestNum  :CARDINAL;

     Keyword  :ARRAY Token OF DynStr;

TYPE BeginTestStatusTyp = (NotBeginTest, TestNumberPending, TestTitlePending);
VAR  BeginTestStatus    :BeginTestStatusTyp;

CONST DEBUG = FALSE;

(*----------------------*)
 PROCEDURE LineFinished;
(*----------------------*)

BEGIN

DynStr1.dStrClr(OneLine);
linePos := 0;

END LineFinished;

(*---------------------*)
 PROCEDURE dRestOfLine;
(*---------------------*)

BEGIN

DynStr2A.dSkipDelims(OneLine, DynStr2A.WhiteSpace, linePos);
DynStr1.dStrSeg(OneLine, linePos, VAL(INTEGER, OneLine.len)-1, OneLine);
DynStr2.dStrCut(OneLine);
linePos := 0;

END dRestOfLine;

(*----------------------------------------*)
 PROCEDURE ReadSkipComments():ReadResults;
(*----------------------------------------*)

VAR  InComment       :BOOLEAN;
     StartCommentPos :INTEGER;
     NumSkipped      :CARDINAL;
     rRes            :ReadResults;

BEGIN

InComment := FALSE;
REPEAT

   IF InComment THEN
      IF (OneLine.len > 1) AND (OneLine.str^[OneLine.len-2] = '*') AND (OneLine.str^[OneLine.len-1] = ')') THEN
         InComment := FALSE;
      END;
   END;

   rRes := dReadNextNonBlankLine(scanfil, OneLine, NumSkipped);
   IF rRes # allRight THEN
      RETURN rRes;
   END;

   INC(curLine, NumSkipped+1);

   DynStr2.dStrCut(OneLine);
   DynStr2A.dStrCollapseLeading(OneLine, DynStr2A.WhiteSpace);

   linePos := 0;
   (*IF DEBUG THEN SWholeIO.WriteCard(curLine, 1);  STextIO.WriteString(OneLine.str^); STextIO.WriteLn; END;*)

   IF NOT InComment THEN
      StartCommentPos := DynStr1.dStr0Pos(OneLine, "(*", 0);
      IF (StartCommentPos = 0) OR ((StartCommentPos = 1) AND (OneLine.str^[0] = ' ')) THEN
         InComment := TRUE;
      END;
   END;

UNTIL NOT InComment;

RETURN rRes;

END ReadSkipComments;

(*---------------------------------------------------------------------*)
 PROCEDURE CheckForToken(VAR tok:Token; VAR linePosNew:INTEGER):BOOLEAN;
(*---------------------------------------------------------------------*)

VAR  TokFound,
     TokNotFound :BOOLEAN;

BEGIN

linePosNew := linePos;

DynStr2A.dNextToken(OneLine, DynStr2A.WhiteSpace, dToken, linePosNew);

TokFound := FALSE;
TokNotFound := FALSE;
tok := MIN(Token);
REPEAT

   IF DynStr1NC.dStrCmpNC(dToken, Keyword[tok]) = 0 THEN
      TokFound := TRUE;
   ELSIF tok < MAX(Token) THEN
      INC(tok);
   ELSE
      TokNotFound := TRUE;
   END;

UNTIL TokFound OR TokNotFound;

RETURN TokFound;

END CheckForToken;

(*====================*)
 PROCEDURE Next_Token;
(*====================*)

VAR  tok         :Token;
     linePosNew  :INTEGER;
     TokFound,
     TokNotFound,
     CheckCol1   :BOOLEAN;
     numstr      :Str0.String16;
     rRes        :ReadResults;

BEGIN

IF BeginTestStatus = TestNumberPending THEN

   INC(TestNum);
   prevTok := curTok;
   curTok := Test_Case_Number;
   CardToStr(TestNum, numstr);
   DynStr3.dStrInE(numstr, dToken);
   BeginTestStatus := TestTitlePending;
   RETURN;                         (* <<<<<<<<<<<<<------------------------  *)

ELSIF BeginTestStatus = TestTitlePending THEN

   dRestOfLine;
   DynStr3.dStrAsgE(OneLine, dToken);
   LineFinished;
   prevTok := curTok;
   curTok := Test_Case_Title;
   BeginTestStatus := NotBeginTest;
   IF dToken.len = 0 THEN
      DynStr3.dStrInE("<notitle>", dToken);
   END;
   RETURN;                         (* <<<<<<<<<<<<<------------------------  *)

END;

IF (OneLine.len > 0) AND (linePos > 0) THEN

   (* must not be a col 1 keyword *)
   CheckCol1 := FALSE;

   dRestOfLine;    (* discards first part of line *)

ELSE

   rRes := ReadSkipComments();

   IF rRes = endOfInput THEN
      prevTok := curTok;
      curTok := End_Of_Input;
      LineFinished;
      DynStr1.dStrClr(dToken);
      RETURN;                         (* <<<<<<<<<<<<<------------------------  *)
   END;

   CheckCol1 := TRUE;
   DynStr2A.dStrCollapse(OneLine, DynStr2A.WhiteSpace); (* tabs to space *)

END;

IF CheckCol1 AND (OneLine.len > 0) AND (OneLine.str^[0] # ' ') THEN

   TokFound := CheckForToken(tok, linePosNew);

   IF TokFound THEN

      linePos := linePosNew;

      CASE tok OF
         Key_Stop, Key_Continue, Key_Exception:      (* these tokens can't be in the col 1 position *)
            TokFound := FALSE;
            TokNotFound := TRUE; |
         Test_Case_Title:
            tok := Key_Begin_Test_Case;
            DynStr1.dStrClr(dToken);
            BeginTestStatus := TestNumberPending; |
         ELSE
      END;

   END;

   prevTok := curTok;
   IF TokFound THEN
      curTok := tok;
   ELSE
      curTok := Error;
   END;

   RETURN;                         (* <<<<<<<<<<<<<------------------------  *)

END;

(* here must be (an empty line or?) not a col 1 line *)

TokFound := CheckForToken(tok, linePosNew);

IF TokFound THEN (* second token on the line *)

   linePos := linePosNew;

   CASE tok OF
      Key_Stop, Key_Continue:
         (*dToken already assigned *)
         IF (curTok = Key_Fail_Handling) OR (curTok = Key_Error_Handling) THEN
            prevTok := curTok;
            curTok := tok;
            LineFinished;
            RETURN;
         ELSE
            tok := Indented_Text_Line;
         END;
      |
      Key_Exception:
         prevTok := curTok;
         curTok := tok;
         RETURN;
      ELSE
         tok := Indented_Text_Line;  (* ? *)
   END;

ELSE

   tok := Indented_Text_Line;

END;

prevTok := curTok;
curTok := tok;
DynStr3.dStrAsgE(OneLine, dToken);
LineFinished;

END Next_Token;

(*===============================*)
 PROCEDURE Current_Token():Token;
(*===============================*)
BEGIN

RETURN curTok;

END Current_Token;

(*=================================*)
 PROCEDURE Current_Line():CARDINAL;
(*=================================*)

BEGIN

RETURN curLine;

END Current_Line;

(*==================================*)
 PROCEDURE Token_Text(VAR t:DynStr);
(*==================================*)

BEGIN

IF t.str = NIL THEN
   DynStr0.dStrAsgC(dToken, t);
ELSE
   DynStr3.dStrAsgE(dToken, t);
END;

END Token_Text;

(*===============================*)
 PROCEDURE InitScanner(f:ChanId);
(*===============================*)

BEGIN

scanfil := f;

IF IsSeqFile(scanfil) THEN   (* unnecessary, but... *)
   Reread(scanfil);
END;

LineFinished;
TestNum := 0;
curLine := 0;
BeginTestStatus := NotBeginTest;

END InitScanner;

(*------------------------------------------------------------*)
 PROCEDURE dStrSetup(VAR ds:DynStr; conststr:DynStr0.DummyStr);
(*------------------------------------------------------------*)
BEGIN

ds.str := conststr;
ds.len := LENGTH(conststr^);
ds.siz := ds.len;

END dStrSetup;

(*-------------------------*)
 BEGIN (* module init code *)
(*-------------------------*)

scanfil := InvalidChan();
DynStr0.dStrCreate(OneLine, 80);
DynStr0.dStrCreate(dToken, 20);
curTok := Error;
prevTok := Error;

dStrSetup(Keyword[Key_Fail_Handling], ADR("fail_handling"));
dStrSetup(Keyword[Key_Error_Handling], ADR("error_handling"));
dStrSetup(Keyword[Key_Stop], ADR("stop"));
dStrSetup(Keyword[Key_Continue], ADR("continue"));

dStrSetup(Keyword[Key_Context], ADR("context"));
(*dStrSetup(Keyword[Key_Exceptions], ADR("exceptions"));*)
dStrSetup(Keyword[Key_Code], ADR("code"));
dStrSetup(Keyword[Key_Begin_Test_Case], ADR(""));

dStrSetup(Keyword[Key_Define], ADR("define"));
dStrSetup(Keyword[Key_Prepare], ADR("prepare"));
dStrSetup(Keyword[Key_Test], ADR("test"));
dStrSetup(Keyword[Key_Pass], ADR("pass"));
dStrSetup(Keyword[Key_Cleanup], ADR("cleanup"));

(*dStrSetup(Keyword[Key_Normal_Path], ADR("normal_path"));*)
dStrSetup(Keyword[Key_Exception], ADR("exception"));
dStrSetup(Keyword[Test_Case_Number], ADR(""));
dStrSetup(Keyword[Test_Case_Title], ADR("*****"));

dStrSetup(Keyword[Indented_Text_Line], ADR(""));
(*dStrSetup(Keyword[Ada_Exception], ADR("ada_exception"));*)
(*dStrSetup(Keyword[Colon], ADR(":"));*)
dStrSetup(Keyword[Semicolon], ADR(";"));
dStrSetup(Keyword[End_Of_Input], ADR(""));
dStrSetup(Keyword[Error], ADR(""));

END tgM2Scanner.
