
(*######################################*)
 IMPLEMENTATION MODULE tgM2TranslatorISO;        (* $VER: tgM2TranslatorISO.mod 0.9 (10.5.04) *)
(*######################################*)
(*
   This file is part of 'tgM2'.

   'tgM2' : A Test Driver Generator for Modula-2 programs
   Copyright (C) 2004 by Tom Breeden.

   'tgM2' is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   'tgM2' is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'tgM2'; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*)
(*
   'tgM2' was patterned on the Ada 95 program, 'tg' which is copyright 1996
    by Andre Spiegel with modifications by Ralf Reissing.
*)

FROM SYSTEM     IMPORT ADDRESS;
FROM Storage    IMPORT ALLOCATE;
FROM Debugging0 IMPORT Assert;

FROM IOChan   IMPORT ChanId, IsChanException, GetName;
FROM TextIO   IMPORT WriteLn, WriteString;
FROM WholeIO  IMPORT WriteCard;
FROM WholeStr IMPORT CardToStr;

FROM tgM2Scanner    IMPORT Current_Line, Current_Token, InitScanner, Next_Token, Token;
FROM tgM2Parser     IMPORT Code_List, Get_Code_Part, General_Part_Type, Get_General_Part,
                           Get_Test_Case, Handling_Mode, Pass_Clause_Type, Path_Type, Test_Case_Type;
FROM tgM2Utils      IMPORT DecrIndent, IncrIndent, New_Line, Positive, Put, Put_ds, Put_Indent,
                           PutIndented, PutIndented_ds, PutIndentedLine, PutIndentedLine_ds, Put_Line,
                           PutNumber, PutQuoted, PutQuoted_ds, SetIndent;
FROM tgM2Exceptions IMPORT IstgException, Raise, tgException, tgExceptions;
IMPORT tgM2Version;

FROM DynStr0             IMPORT DynStr, DynStrPtr;
FROM ListV2Processor     IMPORT List, ListAddition, ListCreate, ListDestroy, ListOpDone, ListRead,
                                ListSetReadMode;
FROM TextBlocks          IMPORT InvalidBlock, NextBlockLineRef, ResetBlockRead;
IMPORT DynStr0, DynStr3, Str0, Str1;

CONST Indent_Step      = 3;

TYPE  ProcCallRec = RECORD
                       Proc_Name :DynStr;
                       testProc  :BOOLEAN;
                    END;
      ProcCallRecPtr = POINTER TO ProcCallRec;
      Call_List_Type = List; (* of ProcCallRecPtr *)

(* NOTE:
     1. Currently, the TextBlocks module has an excess blank line at the end of every TextBlock.
        Here we are working around that simply by ignoring all blank lines in a TextBlock.
*)

(*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*)
 PROCEDURE SeqOrdering(A, B:ADDRESS):INTEGER;
(*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*)

BEGIN

IF A = B THEN
   RETURN 0;
ELSE
   RETURN -1;
END;

END SeqOrdering;

(*==================================================================================*)
 PROCEDURE Translate(Input_File, Output_File, Error_Stream:ChanId;
                     Unit_Name:DynStr; Fail_Output, Pass_Output:Output_Mode);
(*==================================================================================*)

VAR   General_Part     :General_Part_Type;
      Test_Case_Number :Positive;
      tge              :tgExceptions;
      Call_List        :Call_List_Type;   (* list of calls to make in program flow *)
      CS_Count         :CARDINAL;         (* "code" section counter *)

         (*---------------------------------------------*)
          PROCEDURE PutCaseHdr(Test_Case:Test_Case_Type);
         (*---------------------------------------------*)
         BEGIN

         Put_Indent; Put('WriteString("('); PutNumber(Test_Case^.Number);
         Put(') '); PutQuoted_ds(Test_Case^.Title); Put_Line('"); WriteLn;');

         END PutCaseHdr;

         (*-------------------------------------*)
          PROCEDURE Put_IndentedCL(CL:Code_List);
         (*-------------------------------------*)

         VAR  line :DynStrPtr;

         BEGIN

         IF CL # InvalidBlock() THEN

            ResetBlockRead(CL);
            WHILE NextBlockLineRef(CL, line) DO
               IF line^.len > 0 THEN                     (* problem in TextBlocks *)
                  PutIndentedLine_ds(line^);
               END;
            END;

         END;

         END Put_IndentedCL;

         (*-----------------------------------------------------------------------------*)
          PROCEDURE Put_Fail(Test_Case:Test_Case_Type; Comment_Expression:ARRAY OF CHAR);
         (*-----------------------------------------------------------------------------*)
                               (* note that `Comment_Expression' is *)
                               (* _not_ the same as `Comment' in Put_Pass *)  (* ? *)

         BEGIN

         PutIndentedLine('Test_Case_Passed := FALSE;');
         PutIndentedLine('tgM2_Fail_Result := TRUE;');

         CASE Fail_Output OF
            Off: ;
               |
            Numbers:
               Put_Indent; Put('WriteString("('); PutNumber(Test_Case^.Number);
               Put_Line(') FAIL.");');
               |
            Titles:
               IF NOT ((Pass_Output IN OutputModeSet{Titles..Full})
                    AND (Fail_Output IN OutputModeSet{Titles..Full})) THEN
                  PutCaseHdr(Test_Case);
               END;
               PutIndentedLine('WriteString("      ...FAIL."); WriteLn;');
               |
            Full:
               IF NOT ((Pass_Output IN OutputModeSet{Titles..Full})
                     AND (Fail_Output IN OutputModeSet{Titles..Full})) THEN
                  PutCaseHdr(Test_Case);
               END;
               PutIndentedLine('WriteString("      ...FAIL."); WriteLn;');
               PutIndented('WriteString("         ('); PutQuoted(Comment_Expression); Put_Line(')"); WriteLn;');
               |
         END;

         IF General_Part^.Fail_Handling = Stop THEN
            PutIndentedLine('WriteString("Testing aborted"); WriteLn; HALT;');
         END;

         END Put_Fail;

         (*------------------------------------------------------------------*)
          PROCEDURE Put_Pass(Test_Case:Test_Case_Type; Comment:ARRAY OF CHAR);
         (*------------------------------------------------------------------*)
         BEGIN

         CASE Pass_Output OF
            Off: ;
               |
            Numbers :
               Put_Indent; Put('WriteString("('); PutNumber(Test_Case^.Number);
               Put_Line(') pass."); WriteLn;');
               |
            Titles :
               IF NOT ((Pass_Output IN OutputModeSet{Titles..Full})
                     AND (Fail_Output IN OutputModeSet{Titles..Full})) THEN
                  PutCaseHdr(Test_Case);
               END;
               PutIndentedLine('WriteString("      ...pass."); WriteLn;');
               |
            Full :
               IF NOT ((Pass_Output IN OutputModeSet{Titles..Full})
                     AND (Fail_Output IN OutputModeSet{Titles..Full})) THEN
                  PutCaseHdr(Test_Case);
               END;
               PutIndentedLine('WriteString("      ...pass."); WriteLn;');
               PutIndented('WriteString("         ('); PutQuoted(Comment); Put_Line(')"); WriteLn;');
               |
         END;

         END Put_Pass;

         (*-----------------------------------------------------------*)
          PROCEDURE Put_Part(Lines:Code_List; Part_Name:ARRAY OF CHAR);
         (*-----------------------------------------------------------*)
         BEGIN

         PutIndented('(* '); Put(Part_Name); Put_Line(' *)');
         Put_IndentedCL(Lines);
         New_Line;

         END Put_Part;

         (*------------------------------------------------*)
          PROCEDURE Put_Test_Part(Test_Case:Test_Case_Type);
         (*------------------------------------------------*)

         BEGIN

         PutIndentedLine('(* test part *)');
         PutIndentedLine('Test_Case_InTestPart := TRUE;');

         Put_IndentedCL(Test_Case^.Test_Part);
         PutIndentedLine('Test_Case_InTestPart := FALSE;');

         END Put_Test_Part;

         (*---------------------------------------------------------------------------------------------*)
          PROCEDURE Put_Predicate_Check(Test_Case:Test_Case_Type; oneClause:Pass_Clause_Type);
         (*---------------------------------------------------------------------------------------------*)

         VAR  onePred  :DynStrPtr;
              comment  :Str0.String132;

         BEGIN WITH oneClause^ DO

         ResetBlockRead(Predicate);
         Assert(NextBlockLineRef(Predicate, onePred), "No test predicates on list");

         PutIndented('IF '); Put_ds(onePred^);
         WHILE NextBlockLineRef(Predicate, onePred) DO
            IF onePred^.len > 0 THEN                           (* sorry, problem in TextBlocks *)
               New_Line; Put_Indent;
               PutIndented_ds(onePred^);
            END;
         END;
         Put(" "); Put_Line('THEN');
         IncrIndent(Indent_Step);

         Str0.StrAsg("predicate is TRUE", comment);
         IF Path = ExceptionPath THEN
            Str1.StrPre("exception ", comment);
         END;
         Put_Pass(Test_Case, comment);
         DecrIndent(Indent_Step);
         PutIndentedLine('ELSE');
         IncrIndent(Indent_Step);
         Str0.StrAsg("predicate is FALSE", comment);
         IF Path = ExceptionPath THEN
            Str1.StrPre("exception ", comment);
         END;
         Put_Fail(Test_Case, comment);
         DecrIndent(Indent_Step);
         PutIndentedLine('END;');

         END END Put_Predicate_Check;

         (*-----------------------------------------------------------------------*)
          PROCEDURE Put_Result_Part(whichPath:Path_Type; Test_Case:Test_Case_Type);
         (*-----------------------------------------------------------------------*)

         VAR   oneClause :Pass_Clause_Type;

         BEGIN

         New_Line;
         PutIndentedLine('(* result part *)');
         (*PutIndentedLine('Test_Case_Passed := TRUE;');*)
         ListSetReadMode(Test_Case^.Pass_Part);
         ListRead(Test_Case^.Pass_Part, oneClause);
         WHILE ListOpDone(Test_Case^.Pass_Part) DO
            IF oneClause^.Path = whichPath THEN
               Put_Predicate_Check(Test_Case, oneClause);
               New_Line;
            END;
            ListRead(Test_Case^.Pass_Part, oneClause);
         END;

         IF whichPath = NormalPath THEN
            IF Test_Case^.ExceptionRequired THEN
               PutIndentedLine('IF NOT Test_Case_Excepted THEN');
               IncrIndent(Indent_Step);
               Put_Fail(Test_Case, "exception did not occur");
               DecrIndent(Indent_Step);
               PutIndentedLine('END;');
            ELSE
               PutIndentedLine('IF Test_Case_Excepted THEN');
               IncrIndent(Indent_Step);
               Put_Fail(Test_Case, "exception occurred");
               DecrIndent(Indent_Step);
               PutIndentedLine('END;');
            END;
         END;

         END Put_Result_Part;

         (*-------------------*)
          PROCEDURE Put_Header;
         (*-------------------*)

         VAR  OneLine  :Str0.String80;

         BEGIN

         Put_Line('(* ----------------------------------------------------------------------------');
         New_Line;
         Put_Line('Aglet Software');
         Put_Line('tmb@virginia.edu');
         Put_Line('-----------------------------------------------');
         New_Line;
         Put_Line('Test Driver Source');
         New_Line;
         GetName(Output_File, OneLine); Str1.StrPre('   File:       ', OneLine); Put_Line(OneLine);
         GetName(Input_File, OneLine);  Str1.StrPre('   Script:     ', OneLine); Put_Line(OneLine);
         New_Line;
         Put_Line('******************************************************************');
         Put_Line('* This file was automatically generated by the "tgM2" program.   *');
         Put_Line('* Modifications should be made in the corresponding script file. *');
         Put_Line('******************************************************************');
         New_Line;
         Put(tgM2Version.PgmName); Put('   '); Put_Line(tgM2Version.Id);
         New_Line;
         Put_Line('---------------------------------------------------------------------------- *)');
         New_Line;
         SetIndent(0);

         END Put_Header;

         (*-----------------*)
          PROCEDURE Put_Main;
         (*-----------------*)

         VAR  oneCall :ProcCallRecPtr;

         BEGIN

         SetIndent(0);

         New_Line;
         Put_Line('(*--------------*)');
         Put(' '); Put_Line('PROCEDURE Main;');
         Put_Line('(*--------------*)');
         New_Line;
         Put_Line('VAR  i :CARDINAL;');
         New_Line;
         Put_Line('BEGIN');
         New_Line;

         Put_Line('i := 0;');
         ListSetReadMode(Call_List);
         ListRead(Call_List, oneCall);
         WHILE ListOpDone(Call_List) DO

            Put_Line('IF i >= tgM2_CurCallNum THEN');
            IncrIndent(Indent_Step);
            PutIndentedLine('INC(tgM2_CurCallNum);');
            IF oneCall^.testProc THEN
               PutIndentedLine('Test_Case_Excepted := FALSE;');
               PutIndentedLine('Test_Case_Passed := TRUE;');
            END;
            PutIndented_ds(oneCall^.Proc_Name); Put_Line('();');
            DecrIndent(Indent_Step);
            PutIndentedLine('END;');
            PutIndentedLine('INC(i);');
            New_Line;
            DynStr0.dStrDispose(oneCall^.Proc_Name);

            ListRead(Call_List, oneCall);

         END;
         ListDestroy(Call_List, TRUE(*DataAlso*), SIZE(ProcCallRec));

         Put_Line('EXCEPT');
         New_Line;
         Put_Line('tgM2_Unexpected_Error := TRUE;');

         IF General_Part^.Error_Handling = Continue THEN
            PutIndentedLine('WriteString("...error."); WriteLn;');
            PutIndentedLine('RETRY;');
         ELSE
            PutIndentedLine('WriteString("error, testing aborted"); WriteLn; HALT;');
            (*Put_Line('RETURN;');*)
            New_Line;
         END;

         Put_Line('END Main;');

         END Put_Main;

         (*-----------------------*)
          PROCEDURE Put_ModuleInit;
         (*-----------------------*)

         BEGIN

         SetIndent(0);

         New_Line;

         Put_Line('(*--------------------*)');
         Put(' ');Put_Line('BEGIN (* module init *)');
         Put_Line('(*--------------------*)');
         New_Line;

         Put_Line('tgM2_Fail_Result := FALSE;');
         Put_Line('tgM2_Unexpected_Error := FALSE;');
         Put_Line('tgM2_CurCallNum := 0;');
         Put_Line('Test_Case_Passed := FALSE;');
         Put_Line('Test_Case_InTestPart := FALSE;');
         New_Line;

         Put_Line('Main;');

         New_Line;
         Put_Line('WriteLn;');
         Put_Line('WriteString("Total test result: ");');
         Put_Line('IF tgM2_Unexpected_Error THEN');
         IncrIndent(Indent_Step);
         PutIndentedLine('WriteString("ERROR");');
         DecrIndent(Indent_Step);
         Put_Line('ELSIF tgM2_Fail_Result THEN');
         IncrIndent(Indent_Step);
         PutIndentedLine('WriteString("FAIL");');
         DecrIndent(Indent_Step);
         Put_Line('ELSE');
         IncrIndent(Indent_Step);
         PutIndentedLine('WriteString("pass");');
         DecrIndent(Indent_Step);
         Put_Line('END;');
         Put_Line('WriteString("."); WriteLn;');
         Put_Line('WriteLn;');

         New_Line;
         Put('END '); Put_ds(Unit_Name); Put_Line('.');

         END Put_ModuleInit;

         (*-------------------------------*)
          PROCEDURE Translate_General_Part;
         (*-------------------------------*)

         BEGIN

         Get_General_Part(General_Part);
         SetIndent(0);

         Put('MODULE '); Put_ds(Unit_Name); Put_Line(';');
         New_Line;
(*
      Put_Line ("use type Ada.Command_Line.Exit_Status;");
      New_Line;
*)
         PutIndentedLine('FROM STextIO IMPORT WriteLn, WriteString;');
         New_Line;
         Put_IndentedCL(General_Part^.Context);
         New_Line;

         PutIndentedLine('(* Global Data *)');
         PutIndented('VAR ');
         Put_Line('tgM2_Fail_Result           :BOOLEAN;');
         SetIndent(4);
         PutIndentedLine('tgM2_Unexpected_Error      :BOOLEAN;');
         PutIndentedLine('tgM2_CurCallNum            :CARDINAL;');

(*
      Put_Indented ("  Status                : Ada.Command_Line.Exit_Status := 0;");
*)
         New_Line;
         PutIndentedLine('(* Data of last Test Case *)');
         PutIndentedLine('Test_Case_Passed      :BOOLEAN;');
         PutIndentedLine('Test_Case_Excepted    :BOOLEAN;');
         PutIndentedLine('Test_Case_InTestPart  :BOOLEAN;');
         New_Line;

         IF General_Part^.Define_Part # InvalidBlock() THEN
            SetIndent(0);
            PutIndentedLine('(* Global definitions from script *)');
            PutIndentedLine('VAR ');
            SetIndent(4);
            Put_IndentedCL(General_Part^.Define_Part);
            New_Line;
            SetIndent(0);
         END;

         END Translate_General_Part;

         (*------------------------*)
          PROCEDURE Translate_Code;
         (*------------------------*)

         VAR  Code      :Code_List;
              newCodeItem :ProcCallRecPtr;
              NumStr      :Str0.String10;

         BEGIN

         NEW(newCodeItem);
         Assert(newCodeItem # NIL, "Out of memory in Translate_Code");

         WITH newCodeItem^ DO
            DynStr0.dStrInC('Code_Section', Proc_Name);
            CardToStr(CS_Count, NumStr);
            DynStr3.dStr0Cat(NumStr, Proc_Name);
            testProc := FALSE;
         END;

         Code := InvalidBlock();

         Get_Code_Part(Code);
         New_Line;

         INC(CS_Count);

         PutIndented('PROCEDURE '); Put_ds(newCodeItem^.Proc_Name); Put_Line(';');
         New_Line;
         PutIndentedLine('BEGIN');
         IncrIndent(Indent_Step);
         Put_IndentedCL(Code);
         DecrIndent(Indent_Step);
         PutIndented('END '); Put_ds(newCodeItem^.Proc_Name);  Put_Line(';');

         ListAddition(Call_List, newCodeItem);
         Assert(ListOpDone(Call_List), "ListAddition FAIL in Translate_Code");

         END Translate_Code;

         (*-----------------------------*)
          PROCEDURE Translate_Test_Case;
         (*-----------------------------*)

         VAR  Test_Case      :Test_Case_Type;
              newCodeItem    :ProcCallRecPtr;
              TestCaseNumStr :Str0.String10;

         BEGIN

         Test_Case := NIL;

         Get_Test_Case(Test_Case);

         NEW(newCodeItem);
         Assert(newCodeItem # NIL, "Out of memory in Translate_Code");

         WITH newCodeItem^ DO
            DynStr0.dStrInC('Test_Case', Proc_Name);
            CardToStr(Test_Case_Number, TestCaseNumStr);
            DynStr3.dStr0Cat(TestCaseNumStr, Proc_Name);
            testProc := TRUE;
         END;

         (* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  Number parsing has been removed
         IF (Test_Case^.Number # 0) AND (VAL(Positive, Test_Case^.Number) # Test_Case_Number) THEN

            WriteString(Error_Stream, 'tgM2: Warning: test case number "');
            WriteCard(Error_Stream, Test_Case^.Number, 1);
            WriteString(Error_Stream, '" in script file is inconsistent.'); WriteLn(Error_Stream);
            WriteString(Error_Stream, '             The correct number is');
            WriteCard(Error_Stream, Test_Case^.Number, 1);
            WriteString(Error_Stream, ' (this will be used in the testdriver).'); WriteLn(Error_Stream);
            WriteLn(Error_Stream);

            Test_Case^.Number := Test_Case_Number;   (* ? *)
         END;
         <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< *)

         New_Line;
         PutIndentedLine('(*------------------------------------------------------------------------------------*)');
         PutIndented('(* Test Case('); Put(TestCaseNumStr); Put(')  '); Put_ds(Test_Case^.Title); Put_Line('*)');
         PutIndentedLine('(*------------------------------------------------------------------------------------*)');

         PutIndented('PROCEDURE '); Put_ds(newCodeItem^.Proc_Name); Put_Line('();');
         New_Line;

         IF Test_Case^.Define_Part # InvalidBlock() THEN
            PutIndentedLine('VAR ');
            IncrIndent(Indent_Step);
            Put_IndentedCL(Test_Case^.Define_Part);
            DecrIndent(Indent_Step);
            New_Line;
         END;

         PutIndentedLine('BEGIN');
         New_Line;

         PutIndentedLine('IF NOT Test_Case_Excepted THEN');
         IncrIndent(Indent_Step);
         New_Line;

         IF (Pass_Output IN OutputModeSet{Titles..Full}) AND (Fail_Output IN OutputModeSet{Titles..Full}) THEN
            PutCaseHdr(Test_Case);
            New_Line;
         END;

         IF Test_Case^.Prepare_Part # InvalidBlock() THEN
            Put_Part(Test_Case^.Prepare_Part, 'prepare part');
         END;

         Put_Test_Part(Test_Case);

         DecrIndent(Indent_Step);
         New_Line;
         PutIndentedLine('END;');

         Put_Result_Part(NormalPath, Test_Case);

         IF Test_Case^.Cleanup_Part # InvalidBlock() THEN
            Put_Part(Test_Case^.Cleanup_Part, 'cleanup part');
         END;

         New_Line;
         PutIndentedLine('EXCEPT');
         New_Line;

         PutIndentedLine('IF Test_Case_InTestPart THEN');
         IncrIndent(Indent_Step);
         PutIndentedLine('Test_Case_InTestPart := FALSE;');
         PutIndentedLine('Test_Case_Excepted := TRUE;');

         Put_Result_Part(ExceptionPath, Test_Case);

         PutIndentedLine('RETRY;');
         DecrIndent(Indent_Step);
         PutIndentedLine('END;');

         New_Line;
         PutIndentedLine('tgM2_Unexpected_Error := TRUE;');
         IF General_Part^.Error_Handling = Continue THEN
            PutIndentedLine('WriteString("      ...ERROR."); WriteLn;');
            PutIndentedLine('RETURN;');
         ELSE
            PutIndentedLine('WriteString("      ERROR, testing aborted"); WriteLn; HALT;');
         END;

         New_Line;
         PutIndented('END '); Put_ds(newCodeItem^.Proc_Name); Put_Line(';');

         ListAddition(Call_List, newCodeItem);
         Assert(ListOpDone(Call_List), "ListAddition FAIL in Translate_Test_Case");

         END Translate_Test_Case;

BEGIN  (* Translate *)

InitScanner(Input_File);
General_Part := NIL;

Test_Case_Number := 1;
CS_Count := 1;

ListCreate(Call_List, SeqOrdering, FALSE(*Duplicates*));
Assert(ListOpDone(Call_List), "ListCreate Call_List FAIL");

Put_Header;

Next_Token;
Translate_General_Part;
SetIndent(Indent_Step);

LOOP
  CASE Current_Token() OF
    End_Of_Input:
      EXIT;
      |
    Key_Begin_Test_Case:
      Translate_Test_Case;
      INC(Test_Case_Number);
      |
    Key_Code:
      Translate_Code;
      |
    Error:
      Raise(Syntax_Error);
      |
    ELSE
      Raise(Syntax_Error);
  END;
END;

Put_Main;
Put_ModuleInit;

(*-----*)
 EXCEPT
(*-----*)

IF IsChanException() THEN

   Raise(File_Error);   (* when Use_Error | Status_Error | Mode_Error *)

ELSIF IstgException() THEN

   tge := tgException();
   IF (tge = Syntax_Error) OR (tge = Parse_Error) THEN

      WriteString(Error_Stream, "tgM2: Syntax Error at line ");
      WriteCard(Error_Stream, Current_Line(), 1); WriteLn(Error_Stream);
      (*Raise(Syntax_Error);*)
      RETURN;

   ELSIF tge = Semantic_Error THEN

      WriteString(Error_Stream, "tgM2: Semantic Error at line ");
      WriteCard(Error_Stream, Current_Line(), 1); WriteLn(Error_Stream);
      (*raise;*)
      RETURN;

   END;

END;

END Translate;

(*-------------------------*)
 (*BEGIN*) (* module init code *)
(*-------------------------*)

END tgM2TranslatorISO.
